package com.snail.exam.s2dao;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TradeService {

    private static final Log log = LogFactory.getLog(TradeService.class);    
    
    private TradeModel pTradeModel;
    
    public TradeService(TradeModel tradeModel) {
        super();
        
        pTradeModel = tradeModel;
    }
    
    /**
     * 受注処理を行います.
     * <pre>
     * 1.売掛金が与信枠を超えないかどうかを調べます.
     * 2.在庫があるかどうかを調べます
     * 3.在庫の引き当てを行います(Itemから在庫が減らされ、Orderテーブルに一行追加される)
     * 4.Order番号を返却します
     * </pre>
     * @param userId ユーザID
     * @param itemId 商品ID
     * @param amount 個数
     * @return Order番号
     * @throws Exception 受注処理が行えなかった
     */
    public long order(long userId,long itemId,int amount) throws Exception{
        
        if(!pTradeModel.checkCreditFacility(userId)){
            throw new Exception("売掛金が与信枠を超えています");
        }
        
        log.debug("与信チェックOK");
        
        if(!pTradeModel.checkStock(itemId, amount)){
            throw new Exception("在庫が足りません");            
        }
        
        log.debug("在庫チェックOK");
        
        return pTradeModel.setAside(userId, itemId, amount);
    }

}
